-- *****************************************************************************
-- Alekto internal IOs
-- LED MatrixModes: Mode+Delay are only be read if in active state
--
-- (c) Uwe Hanke, HHO_Automation
-- Version $20100316
--         $20100401: switch only differences, read external Delay if localValue=0
--         $20100402: optimize internal object reading
-- *****************************************************************************

-- Table LedMatrix: LedLights l1-l6 0=Off/1=On/8-noChange, rp=repeat
mode = { -- running light around: top > bottom
         { l1 = { 1, 8, 8, 8, 8, 8, 0, 8, 8, 8, 8, 8 },
           l2 = { 8, 1, 8, 8, 8, 8, 8, 0, 8, 8, 8, 8 },
           l3 = { 8, 8, 1, 8, 8, 8, 8, 8, 0, 8, 8, 8 },
           l4 = { 8, 8, 8, 1, 8, 8, 8, 8, 8, 0, 8, 8 },
           l5 = { 8, 8, 8, 8, 1, 8, 8, 8, 8, 8, 0, 8 },
           l6 = { 8, 8, 8, 8, 8, 1, 8, 8, 8, 8, 8, 0 },
           rp = 8 },
         -- running light arround: bottom > top
         { l1 = { 8, 8, 8, 8, 8, 1, 8, 8, 8, 8, 8, 0 },
           l2 = { 8, 8, 8, 8, 1, 8, 8, 8, 8, 8, 0, 8 },
           l3 = { 8, 8, 8, 1, 8, 8, 8, 8, 8, 0, 8, 8 },
           l4 = { 8, 8, 1, 8, 8, 8, 8, 8, 0, 8, 8, 8 },
           l5 = { 8, 1, 8, 8, 8, 8, 8, 0, 8, 8, 8, 8 },
           l6 = { 1, 8, 8, 8, 8, 8, 0, 8, 8, 8, 8, 8 },
           rp = 8 },
         -- running light arround: top > bottom & bottom > top 
         { l1 = { 1, 8, 8, 8, 8, 8, 0, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 1, 8, 8, 8, 8, 8, 0 },
           l2 = { 8, 1, 8, 8, 8, 8, 8, 0, 8, 8, 8, 8, 8, 8, 8, 8, 8, 1, 8, 8, 8, 8, 8, 0, 8 },
           l3 = { 8, 8, 1, 8, 8, 8, 8, 8, 0, 8, 8, 8, 8, 8, 8, 8, 1, 8, 8, 8, 8, 8, 0, 8, 8 },
           l4 = { 8, 8, 8, 1, 8, 8, 8, 8, 8, 0, 8, 8, 8, 8, 8, 1, 8, 8, 8, 8, 8, 0, 8, 8, 8 },
           l5 = { 8, 8, 8, 8, 1, 8, 8, 8, 8, 8, 0, 8, 8, 8, 1, 8, 8, 8, 8, 8, 0, 8, 8, 8, 8 },
           l6 = { 8, 8, 8, 8, 8, 1, 8, 8, 8, 8, 8, 0, 8, 1, 8, 8, 8, 8, 8, 0, 8, 8, 8, 8, 8 },
           rp = 5 },
         -- running light: top > bottom > top
         { l1 = { 1, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 0 },
           l2 = { 8, 1, 8, 8, 8, 8, 8, 8, 8, 8, 0, 8 },
           l3 = { 8, 8, 1, 8, 8, 8, 8, 8, 8, 0, 8, 8 },
           l4 = { 8, 8, 8, 1, 8, 8, 8, 8, 0, 8, 8, 8 },
           l5 = { 8, 8, 8, 8, 1, 8, 8, 0, 8, 8, 8, 8 },
           l6 = { 8, 8, 8, 8, 8, 1, 0, 8, 8, 8, 8, 8 },
           rp = 8 },
         -- running light: bottom > top > bottom
         { l1 = { 8, 8, 8, 8, 8, 1, 0, 8, 8, 8, 8, 8 },
           l2 = { 8, 8, 8, 8, 1, 8, 8, 0, 8, 8, 8, 8 },
           l3 = { 8, 8, 8, 1, 8, 8, 8, 8, 0, 8, 8, 8 },
           l4 = { 8, 8, 1, 8, 8, 8, 8, 8, 8, 0, 8, 8 },
           l5 = { 8, 1, 8, 8, 8, 8, 8, 8, 8, 8, 0, 8 },
           l6 = { 1, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 0 },
           rp = 8 },
         -- running light: top > bottom > top & bottom > top > bottom
         { l1 = { 1, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 0, 8, 8, 8, 8, 8, 8, 1, 0, 8, 8, 8, 8, 8 },
           l2 = { 8, 1, 8, 8, 8, 8, 8, 8, 8, 8, 0, 8, 8, 8, 8, 8, 8, 1, 8, 8, 0, 8, 8, 8, 8 },
           l3 = { 8, 8, 1, 8, 8, 8, 8, 8, 8, 0, 8, 8, 8, 8, 8, 8, 1, 8, 8, 8, 8, 0, 8, 8, 8 },
           l4 = { 8, 8, 8, 1, 8, 8, 8, 8, 0, 8, 8, 8, 8, 8, 8, 1, 8, 8, 8, 8, 8, 8, 0, 8, 8 },
           l5 = { 8, 8, 8, 8, 1, 8, 8, 0, 8, 8, 8, 8, 8, 8, 1, 8, 8, 8, 8, 8, 8, 8, 8, 0, 8 },
           l6 = { 8, 8, 8, 8, 8, 1, 0, 8, 8, 8, 8, 8, 8, 1, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 0 },
           rp = 5 },
         -- running light: top,bottom <> middle & middle <> top,bottom
         { l1 = { 1, 8, 8, 8, 8, 8, 8, 8, 8, 8, 0, 8, 8, 8, 8, 8, 8, 8, 1, 8, 0, 8, 8, 8, 8 },
           l2 = { 8, 8, 1, 8, 8, 8, 8, 8, 0, 8, 8, 8, 8, 8, 8, 8, 1, 8, 8, 8, 8, 8, 0, 8, 8 },
           l3 = { 8, 8, 8, 8, 1, 8, 0, 8, 8, 8, 8, 8, 8, 8, 1, 8, 8, 8, 8, 8, 8, 8, 8, 8, 0 },
           l4 = { 8, 8, 8, 8, 8, 1, 8, 0, 8, 8, 8, 8, 8, 1, 8, 8, 8, 8, 8, 8, 8, 8, 8, 0, 8 },
           l5 = { 8, 8, 8, 1, 8, 8, 8, 8, 8, 0, 8, 8, 8, 8, 8, 1, 8, 8, 8, 8, 8, 0, 8, 8, 8 },
           l6 = { 8, 1, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 0, 8, 8, 8, 8, 1, 8, 0, 8, 8, 8, 8, 8 },
           rp = 6 },
         -- simple running 1 led top > bottom -> top
         { l1 = { 1, 0, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 1, 0 },
           l2 = { 8, 1, 0, 8, 8, 8, 8, 8, 8, 8, 8, 8, 1, 0, 8 },
           l3 = { 8, 8, 1, 0, 8, 8, 8, 8, 8, 8, 8, 1, 0, 8, 8 },
           l4 = { 8, 8, 8, 1, 0, 8, 8, 8, 8, 8, 1, 0, 8, 8, 8 },
           l5 = { 8, 8, 8, 8, 1, 0, 8, 8, 8, 1, 0, 8, 8, 8, 8 },
           l6 = { 8, 8, 8, 8, 8, 1, 0, 8, 1, 0, 8, 8, 8, 8, 8 },
           rp = 5 },
         -- simple running 1 led
         { l1 = { 1, 0, 8, 8, 8, 8, 8 },
           l2 = { 8, 1, 0, 8, 8, 8, 8 },
           l3 = { 8, 8, 1, 0, 8, 8, 8 },
           l4 = { 8, 8, 8, 1, 0, 8, 8 },
           l5 = { 8, 8, 8, 8, 1, 0, 8 },
           l6 = { 8, 8, 8, 8, 8, 1, 0 },
           rp = 10 },
         -- last entry, do not delete
         -- BACnet object references: OnOff,Delay,Mode [objTyp,objId] + ExternalDelay [devId,objTyp,objId,factor]
         { boOnOff = { 3, 107 }, boDelay = { 2, 112 }, boMode = { 19, 111 }, boExtDelay = { 9328, 1, 0, 25.0 }, OnOff = 1, Mode = 1 }
       }

-- *****************************************************************************
-- do not edit from here...
-- *****************************************************************************

Prompt = " DIO-LedMatrix:"

function initleds(value)
   local i
   for i=1,6 do
      err = LuaFieldVarWrite(i, value)
   end
end
function readNoBIntern(typ, id, prop, default)
   err, value = LuaBACnPropReadIntern(typ, id, prop)
   if (err == 0) then
      return tonumber(value)
   end
   return default
end

-- wait @ start
LuaDelay(300)
LuaLogWrite(Prompt, "start...")

-- init
initleds(1); LuaDelay(333); initleds(0)
activ = 1 - readNoBIntern(mode[#mode].boOnOff[1], mode[#mode].boOnOff[2], 85, 0)
delay = readNoBIntern(mode[#mode].boDelay[1], mode[#mode].boDelay[2], 85, 50)
modus = readNoBIntern(mode[#mode].boMode[1], mode[#mode].boMode[2], 85, 1)
index = 1; if (modus > 2 and modus <= #mode) then index = modus - 1; end
count = 1; rpeat = 1; accdelay = 0


-- MainLoop
while (LuaDelay(delay) == 1) do

   if (activ == 1) then
      -- set leds
      if (mode[index].l1[count] <= 1) then err = LuaFieldVarWrite(1, mode[index].l1[count]); end
      if (mode[index].l2[count] <= 1) then err = LuaFieldVarWrite(2, mode[index].l2[count]); end
      if (mode[index].l3[count] <= 1) then err = LuaFieldVarWrite(3, mode[index].l3[count]); end
      if (mode[index].l4[count] <= 1) then err = LuaFieldVarWrite(4, mode[index].l4[count]); end
      if (mode[index].l5[count] <= 1) then err = LuaFieldVarWrite(5, mode[index].l5[count]); end
      if (mode[index].l6[count] <= 1) then err = LuaFieldVarWrite(6, mode[index].l6[count]); end
   end

   -- check OnOff, ca. 200ms
   accdelay = accdelay + delay
   if (delay < 11) then accdelay = accdelay + 11; end
   if (accdelay >= 200) then
      accdelay = 0
      activ = 1 - readNoBIntern(mode[#mode].boOnOff[1], mode[#mode].boOnOff[2], 85, 0)
      if (activ ~= mode[#mode].OnOff) then
         mode[#mode].OnOff = activ
         initleds(activ); LuaDelay(333)
         rpeat = mode[index].rp
         count = #mode[index].l1
      end
   end

   if (activ == 1) then
      -- counter increment
      count = count + 1
      if (count > #mode[index].l1) then    -- cycle complete
         count = 1
         -- repeater increment
         rpeat = rpeat + 1
         if (rpeat > mode[index].rp) then  -- Mode complete
            rpeat = 1
            -- check Delay
            delay = readNoBIntern(mode[#mode].boDelay[1], mode[#mode].boDelay[2], 85, 50)
            if (delay == 0) then    -- read from extern
               delay = 50
               err, val = LuaBACnPropRead(mode[#mode].boExtDelay[1], mode[#mode].boExtDelay[2], mode[#mode].boExtDelay[3], 85)
               if (err == 0) then
                  delay = tonumber(val) * mode[#mode].boExtDelay[4] -- factor value
                  LuaBACnPropWriteIntern(mode[#mode].boDelay[1], mode[#mode].boDelay[2], 104, delay)    -- write relinquishDefault
               end
            end
            -- check Mode
            modus = readNoBIntern(mode[#mode].boMode[1], mode[#mode].boMode[2], 85, 1)
            if (modus ~= mode[#mode].Mode) then
               mode[#mode].Mode = modus
               index = #mode - 1
            end
            -- index increment, if Automode
            if (modus == 1) then
               index = index + 1
               if (index >= #mode) then
                  index = 1
               end
            elseif (modus >= 2 and modus <= #mode) then 
               index = modus - 1
            end
            -- blinking
            LuaDelay(222); initleds(1); LuaDelay(222); initleds(0); LuaDelay(222)
         end
      end
   end

end

-- end
initleds(0)
LuaLogWrite(Prompt, "...end")
