-- *****************************************************************************
-- Mapping COV-Messages to local/remote Objects: propertyValue = covValue
-- exemplary Thermokon MS/TP SRC65-Profiles
--
-- (c) Uwe Hanke, HHO_Automation
-- Version $20100316, $20101106
-- *****************************************************************************

-- Table COV Reference, only edit this one
-- tableStringIndex COV-DeviceObjectPropertyReference = local/remote DeviceObjectPropertyReference, same propertyValue
--
--  Format: ["deviceId,objectTyp,objectId,propertyId"] = { devId = Id, objTyp = Typ, objId = Id, propId = Id, prio = Prio }
--            device = -1 for mapping local objects @ EDE_objects file with internal writing prio 0
--
covref = {  -- EnOcean SRC65 Device 9201 -> local Device:
            -- EasyFit-EasySens-4-Kanal-Taster, Kanal 1
           ["9201,3,100,85"]  = { devId = -1, objTyp = 3,  objId = 1201, propId = 85, prio = 0 },
           ["9201,3,101,85"]  = { devId = -1, objTyp = 3,  objId = 1202, propId = 85, prio = 0 },
           ["9201,3,102,85"]  = { devId = -1, objTyp = 3,  objId = 1203, propId = 85, prio = 0 },
           ["9201,3,103,85"]  = { devId = -1, objTyp = 3,  objId = 1204, propId = 85, prio = 0 },
            -- Raumbediengerät PST, Kanal 2
           ["9201,0,201,85"]  = { devId = -1, objTyp = 0,  objId = 1205, propId = 85, prio = 0 },
           ["9201,0,202,85"]  = { devId = -1, objTyp = 0,  objId = 1206, propId = 85, prio = 0 },
           ["9201,13,200,85"] = { devId = -1, objTyp = 13, objId = 1207, propId = 85, prio = 0 },
           ["9201,3,200,85"]  = { devId = -1, objTyp = 3,  objId = 1208, propId = 85, prio = 0 },
           --["9201,3,200,85"]  = { devId = 2098177, objTyp = 4,  objId = 12, propId = 85, prio = 8 },  -- Siemens
            -- last entry (do not delete): 
            -- Programmstate as internal MultiState 1-Stop/2-Run/3-Error
           ["programState"]   = { oldval = 1, objTyp = 13, objId = 1200, propId = 85 }
         }

--
-- *****************************************************************************
-- do not edit from here...
-- *****************************************************************************

Prompt = " COV2LRObjects:"

-- functions: set programSate
function setPState(val)
    local idx = "programState"
    if (covref[idx] and covref[idx].oldval ~= val) then
        LuaBACnPropWriteIntern(covref[idx].objTyp, covref[idx].objId, covref[idx].propId, val)
        covref[idx].oldval = val
    end
end
-- write local/remote value
function writeBACValue(dev, typ, idx, prop, prio, val)
    -- internal/remote objects
    if (dev == -1) then
        LuaBACnPropWriteIntern(typ, idx, prop, val)
    else
        err = LuaBACnPropWrite(dev, typ, idx, prop, prio, val)
        if (err ~= 0) then
            errClass, errCode = LuaBACnGetLastError()
            LuaLogWrite(Prompt, string.format("Error write remote object [%d,%d,%d,%d]: %d/%d", dev, typ, idx, prop, errClass, errCode))
            setPState(3)
            return
        end
    end
    setPState(2)
end
-- init (read+write) all values
function initAllValues()
    for ref in pairs(covref) do
        if (ref ~= "programState") then
            dev, typ, idx, prop = string.match(ref, "(%d+),(%d+),(%d+),(%d+)")
            err, val = LuaBACnPropRead(dev, typ, idx, prop)
            if (err == 0) then
                writeBACValue(covref[ref].devId, covref[ref].objTyp, covref[ref].objId, covref[ref].propId, covref[ref].prio, val)
            else
                errClass, errCode = LuaBACnGetLastError()
                LuaLogWrite(Prompt, string.format("Error reading object [%d,%d,%d,%d]: %d/%d", dev, typ, idx, prop, errClass, errCode))
                setPState(3)
            end
        end
    end
end
-- read internal object property as number value
function readNoBIntern(typ, id, prop, default)
   err, value = LuaBACnPropReadIntern(typ, id, prop)
   if (err == 0) then
      return tonumber(value)
   end
   return default
end


-- wait 3s @ start
LuaDelay(3000)
LuaLogWrite(Prompt, "start...")
setPState(2)

-- read all values & clean COV-Buffer
LuaLogWrite(Prompt, "initialize all referenced objects..")
initAllValues()
LuaCOVBufferClear()
LuaLogWrite(Prompt, "buffer cleared, ready for COV..")
setPState(2)

-- MainLoop
while (LuaDelay(5) == 1) do

    count = 0
    repeat
        -- read Buffer
        berr, bref, bval = LuaCOVBufferRead()
        if (berr ~= 0) then
            -- debugprintlog
            LuaDebugTask(Prompt, string.format("Debug cov-message received [%s] = %s", bref, bval))
            -- buffer overflow
            if (berr == -1) then
                LuaLogWrite(Prompt, "Error COV-Buffer overflow detected!")
                setPState(3)
            end
            -- check reference & write mapping
            if (covref[bref]) then
                -- debugprintlog
                LuaDebugTask(Prompt, string.format("Debug mapping found, write object [%d,%d,%d,%d] = %s", covref[bref].devId, covref[bref].objTyp, covref[bref].objId, covref[bref].propId, bval))
                writeBACValue(covref[bref].devId, covref[bref].objTyp, covref[bref].objId, covref[bref].propId, covref[bref].prio, bval)
            end
            -- buffer read max @ one loop
            count = count + 1
            if (count >= 32) then break; end
        end
    until (berr == 0)

end

-- write Status 1=Stop & end
setPState(1)
LuaLogWrite(Prompt, "...end")
